"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class CreateReq(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Options(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LOGIN_NAME_FIELD_NUMBER: builtins.int
        PASSWORD_FIELD_NUMBER: builtins.int
        FULL_NAME_FIELD_NUMBER: builtins.int
        GROUPS_FIELD_NUMBER: builtins.int
        login_name: builtins.str
        password: builtins.str
        full_name: builtins.str
        @property
        def groups(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
        def __init__(
            self,
            *,
            login_name: builtins.str = ...,
            password: builtins.str = ...,
            full_name: builtins.str = ...,
            groups: collections.abc.Iterable[builtins.str] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["full_name", b"full_name", "groups", b"groups", "login_name", b"login_name", "password", b"password"]) -> None: ...

    OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> global___CreateReq.Options: ...
    def __init__(
        self,
        *,
        options: global___CreateReq.Options | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["options", b"options"]) -> None: ...

global___CreateReq = CreateReq

class CreateResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___CreateResp = CreateResp

class UpdateReq(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Options(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LOGIN_NAME_FIELD_NUMBER: builtins.int
        PASSWORD_FIELD_NUMBER: builtins.int
        FULL_NAME_FIELD_NUMBER: builtins.int
        GROUPS_FIELD_NUMBER: builtins.int
        login_name: builtins.str
        password: builtins.str
        full_name: builtins.str
        @property
        def groups(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
        def __init__(
            self,
            *,
            login_name: builtins.str = ...,
            password: builtins.str = ...,
            full_name: builtins.str = ...,
            groups: collections.abc.Iterable[builtins.str] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["full_name", b"full_name", "groups", b"groups", "login_name", b"login_name", "password", b"password"]) -> None: ...

    OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> global___UpdateReq.Options: ...
    def __init__(
        self,
        *,
        options: global___UpdateReq.Options | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["options", b"options"]) -> None: ...

global___UpdateReq = UpdateReq

class UpdateResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___UpdateResp = UpdateResp

class DeleteReq(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Options(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LOGIN_NAME_FIELD_NUMBER: builtins.int
        login_name: builtins.str
        def __init__(
            self,
            *,
            login_name: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["login_name", b"login_name"]) -> None: ...

    OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> global___DeleteReq.Options: ...
    def __init__(
        self,
        *,
        options: global___DeleteReq.Options | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["options", b"options"]) -> None: ...

global___DeleteReq = DeleteReq

class DeleteResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DeleteResp = DeleteResp

class EnableReq(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Options(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LOGIN_NAME_FIELD_NUMBER: builtins.int
        login_name: builtins.str
        def __init__(
            self,
            *,
            login_name: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["login_name", b"login_name"]) -> None: ...

    OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> global___EnableReq.Options: ...
    def __init__(
        self,
        *,
        options: global___EnableReq.Options | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["options", b"options"]) -> None: ...

global___EnableReq = EnableReq

class EnableResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___EnableResp = EnableResp

class DisableReq(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Options(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LOGIN_NAME_FIELD_NUMBER: builtins.int
        login_name: builtins.str
        def __init__(
            self,
            *,
            login_name: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["login_name", b"login_name"]) -> None: ...

    OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> global___DisableReq.Options: ...
    def __init__(
        self,
        *,
        options: global___DisableReq.Options | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["options", b"options"]) -> None: ...

global___DisableReq = DisableReq

class DisableResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DisableResp = DisableResp

class DetailsReq(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Options(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LOGIN_NAME_FIELD_NUMBER: builtins.int
        login_name: builtins.str
        def __init__(
            self,
            *,
            login_name: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["login_name", b"login_name"]) -> None: ...

    OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> global___DetailsReq.Options: ...
    def __init__(
        self,
        *,
        options: global___DetailsReq.Options | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["options", b"options"]) -> None: ...

global___DetailsReq = DetailsReq

class DetailsResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class UserDetails(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class DateTime(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            TICKS_SINCE_EPOCH_FIELD_NUMBER: builtins.int
            ticks_since_epoch: builtins.int
            def __init__(
                self,
                *,
                ticks_since_epoch: builtins.int = ...,
            ) -> None: ...
            def ClearField(self, field_name: typing_extensions.Literal["ticks_since_epoch", b"ticks_since_epoch"]) -> None: ...

        LOGIN_NAME_FIELD_NUMBER: builtins.int
        FULL_NAME_FIELD_NUMBER: builtins.int
        GROUPS_FIELD_NUMBER: builtins.int
        LAST_UPDATED_FIELD_NUMBER: builtins.int
        DISABLED_FIELD_NUMBER: builtins.int
        login_name: builtins.str
        full_name: builtins.str
        @property
        def groups(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
        @property
        def last_updated(self) -> global___DetailsResp.UserDetails.DateTime: ...
        disabled: builtins.bool
        def __init__(
            self,
            *,
            login_name: builtins.str = ...,
            full_name: builtins.str = ...,
            groups: collections.abc.Iterable[builtins.str] | None = ...,
            last_updated: global___DetailsResp.UserDetails.DateTime | None = ...,
            disabled: builtins.bool = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["last_updated", b"last_updated"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["disabled", b"disabled", "full_name", b"full_name", "groups", b"groups", "last_updated", b"last_updated", "login_name", b"login_name"]) -> None: ...

    USER_DETAILS_FIELD_NUMBER: builtins.int
    @property
    def user_details(self) -> global___DetailsResp.UserDetails: ...
    def __init__(
        self,
        *,
        user_details: global___DetailsResp.UserDetails | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["user_details", b"user_details"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["user_details", b"user_details"]) -> None: ...

global___DetailsResp = DetailsResp

class ChangePasswordReq(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Options(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LOGIN_NAME_FIELD_NUMBER: builtins.int
        CURRENT_PASSWORD_FIELD_NUMBER: builtins.int
        NEW_PASSWORD_FIELD_NUMBER: builtins.int
        login_name: builtins.str
        current_password: builtins.str
        new_password: builtins.str
        def __init__(
            self,
            *,
            login_name: builtins.str = ...,
            current_password: builtins.str = ...,
            new_password: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["current_password", b"current_password", "login_name", b"login_name", "new_password", b"new_password"]) -> None: ...

    OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> global___ChangePasswordReq.Options: ...
    def __init__(
        self,
        *,
        options: global___ChangePasswordReq.Options | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["options", b"options"]) -> None: ...

global___ChangePasswordReq = ChangePasswordReq

class ChangePasswordResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ChangePasswordResp = ChangePasswordResp

class ResetPasswordReq(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Options(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LOGIN_NAME_FIELD_NUMBER: builtins.int
        NEW_PASSWORD_FIELD_NUMBER: builtins.int
        login_name: builtins.str
        new_password: builtins.str
        def __init__(
            self,
            *,
            login_name: builtins.str = ...,
            new_password: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["login_name", b"login_name", "new_password", b"new_password"]) -> None: ...

    OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> global___ResetPasswordReq.Options: ...
    def __init__(
        self,
        *,
        options: global___ResetPasswordReq.Options | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["options", b"options"]) -> None: ...

global___ResetPasswordReq = ResetPasswordReq

class ResetPasswordResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ResetPasswordResp = ResetPasswordResp



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: serverfeatures.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14serverfeatures.proto\x12\"event_store.client.server_features\x1a\x0cshared.proto\"|\n\x10SupportedMethods\x12\x44\n\x07methods\x18\x01 \x03(\x0b\x32\x33.event_store.client.server_features.SupportedMethod\x12\"\n\x1a\x65vent_store_server_version\x18\x02 \x01(\t\"N\n\x0fSupportedMethod\x12\x13\n\x0bmethod_name\x18\x01 \x01(\t\x12\x14\n\x0cservice_name\x18\x02 \x01(\t\x12\x10\n\x08\x66\x65\x61tures\x18\x03 \x03(\t2x\n\x0eServerFeatures\x12\x66\n\x13GetSupportedMethods\x12\x19.event_store.client.Empty\x1a\x34.event_store.client.server_features.SupportedMethodsB.\n,com.eventstore.dbclient.proto.serverfeaturesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'serverfeatures_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n,com.eventstore.dbclient.proto.serverfeatures'
  _SUPPORTEDMETHODS._serialized_start=74
  _SUPPORTEDMETHODS._serialized_end=198
  _SUPPORTEDMETHOD._serialized_start=200
  _SUPPORTEDMETHOD._serialized_end=278
  _SERVERFEATURES._serialized_start=280
  _SERVERFEATURES._serialized_end=400
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import persistent_pb2 as persistent__pb2
from . import shared_pb2 as shared__pb2


class PersistentSubscriptionsStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Create = channel.unary_unary(
                '/event_store.client.persistent_subscriptions.PersistentSubscriptions/Create',
                request_serializer=persistent__pb2.CreateReq.SerializeToString,
                response_deserializer=persistent__pb2.CreateResp.FromString,
                )
        self.Update = channel.unary_unary(
                '/event_store.client.persistent_subscriptions.PersistentSubscriptions/Update',
                request_serializer=persistent__pb2.UpdateReq.SerializeToString,
                response_deserializer=persistent__pb2.UpdateResp.FromString,
                )
        self.Delete = channel.unary_unary(
                '/event_store.client.persistent_subscriptions.PersistentSubscriptions/Delete',
                request_serializer=persistent__pb2.DeleteReq.SerializeToString,
                response_deserializer=persistent__pb2.DeleteResp.FromString,
                )
        self.Read = channel.stream_stream(
                '/event_store.client.persistent_subscriptions.PersistentSubscriptions/Read',
                request_serializer=persistent__pb2.ReadReq.SerializeToString,
                response_deserializer=persistent__pb2.ReadResp.FromString,
                )
        self.GetInfo = channel.unary_unary(
                '/event_store.client.persistent_subscriptions.PersistentSubscriptions/GetInfo',
                request_serializer=persistent__pb2.GetInfoReq.SerializeToString,
                response_deserializer=persistent__pb2.GetInfoResp.FromString,
                )
        self.ReplayParked = channel.unary_unary(
                '/event_store.client.persistent_subscriptions.PersistentSubscriptions/ReplayParked',
                request_serializer=persistent__pb2.ReplayParkedReq.SerializeToString,
                response_deserializer=persistent__pb2.ReplayParkedResp.FromString,
                )
        self.List = channel.unary_unary(
                '/event_store.client.persistent_subscriptions.PersistentSubscriptions/List',
                request_serializer=persistent__pb2.ListReq.SerializeToString,
                response_deserializer=persistent__pb2.ListResp.FromString,
                )
        self.RestartSubsystem = channel.unary_unary(
                '/event_store.client.persistent_subscriptions.PersistentSubscriptions/RestartSubsystem',
                request_serializer=shared__pb2.Empty.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )


class PersistentSubscriptionsServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Update(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Delete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Read(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetInfo(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReplayParked(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RestartSubsystem(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PersistentSubscriptionsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Create': grpc.unary_unary_rpc_method_handler(
                    servicer.Create,
                    request_deserializer=persistent__pb2.CreateReq.FromString,
                    response_serializer=persistent__pb2.CreateResp.SerializeToString,
            ),
            'Update': grpc.unary_unary_rpc_method_handler(
                    servicer.Update,
                    request_deserializer=persistent__pb2.UpdateReq.FromString,
                    response_serializer=persistent__pb2.UpdateResp.SerializeToString,
            ),
            'Delete': grpc.unary_unary_rpc_method_handler(
                    servicer.Delete,
                    request_deserializer=persistent__pb2.DeleteReq.FromString,
                    response_serializer=persistent__pb2.DeleteResp.SerializeToString,
            ),
            'Read': grpc.stream_stream_rpc_method_handler(
                    servicer.Read,
                    request_deserializer=persistent__pb2.ReadReq.FromString,
                    response_serializer=persistent__pb2.ReadResp.SerializeToString,
            ),
            'GetInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetInfo,
                    request_deserializer=persistent__pb2.GetInfoReq.FromString,
                    response_serializer=persistent__pb2.GetInfoResp.SerializeToString,
            ),
            'ReplayParked': grpc.unary_unary_rpc_method_handler(
                    servicer.ReplayParked,
                    request_deserializer=persistent__pb2.ReplayParkedReq.FromString,
                    response_serializer=persistent__pb2.ReplayParkedResp.SerializeToString,
            ),
            'List': grpc.unary_unary_rpc_method_handler(
                    servicer.List,
                    request_deserializer=persistent__pb2.ListReq.FromString,
                    response_serializer=persistent__pb2.ListResp.SerializeToString,
            ),
            'RestartSubsystem': grpc.unary_unary_rpc_method_handler(
                    servicer.RestartSubsystem,
                    request_deserializer=shared__pb2.Empty.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'event_store.client.persistent_subscriptions.PersistentSubscriptions', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class PersistentSubscriptions(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Create(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.persistent_subscriptions.PersistentSubscriptions/Create',
            persistent__pb2.CreateReq.SerializeToString,
            persistent__pb2.CreateResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Update(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.persistent_subscriptions.PersistentSubscriptions/Update',
            persistent__pb2.UpdateReq.SerializeToString,
            persistent__pb2.UpdateResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Delete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.persistent_subscriptions.PersistentSubscriptions/Delete',
            persistent__pb2.DeleteReq.SerializeToString,
            persistent__pb2.DeleteResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Read(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/event_store.client.persistent_subscriptions.PersistentSubscriptions/Read',
            persistent__pb2.ReadReq.SerializeToString,
            persistent__pb2.ReadResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.persistent_subscriptions.PersistentSubscriptions/GetInfo',
            persistent__pb2.GetInfoReq.SerializeToString,
            persistent__pb2.GetInfoResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReplayParked(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.persistent_subscriptions.PersistentSubscriptions/ReplayParked',
            persistent__pb2.ReplayParkedReq.SerializeToString,
            persistent__pb2.ReplayParkedResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def List(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.persistent_subscriptions.PersistentSubscriptions/List',
            persistent__pb2.ListReq.SerializeToString,
            persistent__pb2.ListResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RestartSubsystem(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.persistent_subscriptions.PersistentSubscriptions/RestartSubsystem',
            shared__pb2.Empty.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import operations_pb2 as operations__pb2
from . import shared_pb2 as shared__pb2


class OperationsStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.StartScavenge = channel.unary_unary(
                '/event_store.client.operations.Operations/StartScavenge',
                request_serializer=operations__pb2.StartScavengeReq.SerializeToString,
                response_deserializer=operations__pb2.ScavengeResp.FromString,
                )
        self.StopScavenge = channel.unary_unary(
                '/event_store.client.operations.Operations/StopScavenge',
                request_serializer=operations__pb2.StopScavengeReq.SerializeToString,
                response_deserializer=operations__pb2.ScavengeResp.FromString,
                )
        self.Shutdown = channel.unary_unary(
                '/event_store.client.operations.Operations/Shutdown',
                request_serializer=shared__pb2.Empty.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )
        self.MergeIndexes = channel.unary_unary(
                '/event_store.client.operations.Operations/MergeIndexes',
                request_serializer=shared__pb2.Empty.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )
        self.ResignNode = channel.unary_unary(
                '/event_store.client.operations.Operations/ResignNode',
                request_serializer=shared__pb2.Empty.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )
        self.SetNodePriority = channel.unary_unary(
                '/event_store.client.operations.Operations/SetNodePriority',
                request_serializer=operations__pb2.SetNodePriorityReq.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )
        self.RestartPersistentSubscriptions = channel.unary_unary(
                '/event_store.client.operations.Operations/RestartPersistentSubscriptions',
                request_serializer=shared__pb2.Empty.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )


class OperationsServicer(object):
    """Missing associated documentation comment in .proto file."""

    def StartScavenge(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopScavenge(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Shutdown(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def MergeIndexes(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResignNode(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetNodePriority(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RestartPersistentSubscriptions(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_OperationsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'StartScavenge': grpc.unary_unary_rpc_method_handler(
                    servicer.StartScavenge,
                    request_deserializer=operations__pb2.StartScavengeReq.FromString,
                    response_serializer=operations__pb2.ScavengeResp.SerializeToString,
            ),
            'StopScavenge': grpc.unary_unary_rpc_method_handler(
                    servicer.StopScavenge,
                    request_deserializer=operations__pb2.StopScavengeReq.FromString,
                    response_serializer=operations__pb2.ScavengeResp.SerializeToString,
            ),
            'Shutdown': grpc.unary_unary_rpc_method_handler(
                    servicer.Shutdown,
                    request_deserializer=shared__pb2.Empty.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
            'MergeIndexes': grpc.unary_unary_rpc_method_handler(
                    servicer.MergeIndexes,
                    request_deserializer=shared__pb2.Empty.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
            'ResignNode': grpc.unary_unary_rpc_method_handler(
                    servicer.ResignNode,
                    request_deserializer=shared__pb2.Empty.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
            'SetNodePriority': grpc.unary_unary_rpc_method_handler(
                    servicer.SetNodePriority,
                    request_deserializer=operations__pb2.SetNodePriorityReq.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
            'RestartPersistentSubscriptions': grpc.unary_unary_rpc_method_handler(
                    servicer.RestartPersistentSubscriptions,
                    request_deserializer=shared__pb2.Empty.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'event_store.client.operations.Operations', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Operations(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def StartScavenge(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.operations.Operations/StartScavenge',
            operations__pb2.StartScavengeReq.SerializeToString,
            operations__pb2.ScavengeResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StopScavenge(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.operations.Operations/StopScavenge',
            operations__pb2.StopScavengeReq.SerializeToString,
            operations__pb2.ScavengeResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Shutdown(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.operations.Operations/Shutdown',
            shared__pb2.Empty.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def MergeIndexes(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.operations.Operations/MergeIndexes',
            shared__pb2.Empty.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResignNode(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.operations.Operations/ResignNode',
            shared__pb2.Empty.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetNodePriority(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.operations.Operations/SetNodePriority',
            operations__pb2.SetNodePriorityReq.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RestartPersistentSubscriptions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.operations.Operations/RestartPersistentSubscriptions',
            shared__pb2.Empty.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import shared_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ClusterInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEMBERS_FIELD_NUMBER: builtins.int
    @property
    def members(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___MemberInfo]: ...
    def __init__(
        self,
        *,
        members: collections.abc.Iterable[global___MemberInfo] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["members", b"members"]) -> None: ...

global___ClusterInfo = ClusterInfo

class EndPoint(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: builtins.int
    PORT_FIELD_NUMBER: builtins.int
    address: builtins.str
    port: builtins.int
    def __init__(
        self,
        *,
        address: builtins.str = ...,
        port: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["address", b"address", "port", b"port"]) -> None: ...

global___EndPoint = EndPoint

class MemberInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _VNodeState:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _VNodeStateEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[MemberInfo._VNodeState.ValueType], builtins.type):  # noqa: F821
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        Initializing: MemberInfo._VNodeState.ValueType  # 0
        DiscoverLeader: MemberInfo._VNodeState.ValueType  # 1
        Unknown: MemberInfo._VNodeState.ValueType  # 2
        PreReplica: MemberInfo._VNodeState.ValueType  # 3
        CatchingUp: MemberInfo._VNodeState.ValueType  # 4
        Clone: MemberInfo._VNodeState.ValueType  # 5
        Follower: MemberInfo._VNodeState.ValueType  # 6
        PreLeader: MemberInfo._VNodeState.ValueType  # 7
        Leader: MemberInfo._VNodeState.ValueType  # 8
        Manager: MemberInfo._VNodeState.ValueType  # 9
        ShuttingDown: MemberInfo._VNodeState.ValueType  # 10
        Shutdown: MemberInfo._VNodeState.ValueType  # 11
        ReadOnlyLeaderless: MemberInfo._VNodeState.ValueType  # 12
        PreReadOnlyReplica: MemberInfo._VNodeState.ValueType  # 13
        ReadOnlyReplica: MemberInfo._VNodeState.ValueType  # 14
        ResigningLeader: MemberInfo._VNodeState.ValueType  # 15

    class VNodeState(_VNodeState, metaclass=_VNodeStateEnumTypeWrapper): ...
    Initializing: MemberInfo.VNodeState.ValueType  # 0
    DiscoverLeader: MemberInfo.VNodeState.ValueType  # 1
    Unknown: MemberInfo.VNodeState.ValueType  # 2
    PreReplica: MemberInfo.VNodeState.ValueType  # 3
    CatchingUp: MemberInfo.VNodeState.ValueType  # 4
    Clone: MemberInfo.VNodeState.ValueType  # 5
    Follower: MemberInfo.VNodeState.ValueType  # 6
    PreLeader: MemberInfo.VNodeState.ValueType  # 7
    Leader: MemberInfo.VNodeState.ValueType  # 8
    Manager: MemberInfo.VNodeState.ValueType  # 9
    ShuttingDown: MemberInfo.VNodeState.ValueType  # 10
    Shutdown: MemberInfo.VNodeState.ValueType  # 11
    ReadOnlyLeaderless: MemberInfo.VNodeState.ValueType  # 12
    PreReadOnlyReplica: MemberInfo.VNodeState.ValueType  # 13
    ReadOnlyReplica: MemberInfo.VNodeState.ValueType  # 14
    ResigningLeader: MemberInfo.VNodeState.ValueType  # 15

    INSTANCE_ID_FIELD_NUMBER: builtins.int
    TIME_STAMP_FIELD_NUMBER: builtins.int
    STATE_FIELD_NUMBER: builtins.int
    IS_ALIVE_FIELD_NUMBER: builtins.int
    HTTP_END_POINT_FIELD_NUMBER: builtins.int
    @property
    def instance_id(self) -> shared_pb2.UUID: ...
    time_stamp: builtins.int
    state: global___MemberInfo.VNodeState.ValueType
    is_alive: builtins.bool
    @property
    def http_end_point(self) -> global___EndPoint: ...
    def __init__(
        self,
        *,
        instance_id: shared_pb2.UUID | None = ...,
        time_stamp: builtins.int = ...,
        state: global___MemberInfo.VNodeState.ValueType = ...,
        is_alive: builtins.bool = ...,
        http_end_point: global___EndPoint | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["http_end_point", b"http_end_point", "instance_id", b"instance_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["http_end_point", b"http_end_point", "instance_id", b"instance_id", "is_alive", b"is_alive", "state", b"state", "time_stamp", b"time_stamp"]) -> None: ...

global___MemberInfo = MemberInfo



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import monitoring_pb2 as monitoring__pb2


class MonitoringStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Stats = channel.unary_stream(
                '/event_store.client.monitoring.Monitoring/Stats',
                request_serializer=monitoring__pb2.StatsReq.SerializeToString,
                response_deserializer=monitoring__pb2.StatsResp.FromString,
                )


class MonitoringServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Stats(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MonitoringServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Stats': grpc.unary_stream_rpc_method_handler(
                    servicer.Stats,
                    request_deserializer=monitoring__pb2.StatsReq.FromString,
                    response_serializer=monitoring__pb2.StatsResp.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'event_store.client.monitoring.Monitoring', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Monitoring(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Stats(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/event_store.client.monitoring.Monitoring/Stats',
            monitoring__pb2.StatsReq.SerializeToString,
            monitoring__pb2.StatsResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gossip.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cgossip.proto\x12\x19\x65vent_store.client.gossip\x1a\x0cshared.proto\"E\n\x0b\x43lusterInfo\x12\x36\n\x07members\x18\x01 \x03(\x0b\x32%.event_store.client.gossip.MemberInfo\")\n\x08\x45ndPoint\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\r\"\xfc\x03\n\nMemberInfo\x12-\n\x0binstance_id\x18\x01 \x01(\x0b\x32\x18.event_store.client.UUID\x12\x12\n\ntime_stamp\x18\x02 \x01(\x03\x12?\n\x05state\x18\x03 \x01(\x0e\x32\x30.event_store.client.gossip.MemberInfo.VNodeState\x12\x10\n\x08is_alive\x18\x04 \x01(\x08\x12;\n\x0ehttp_end_point\x18\x05 \x01(\x0b\x32#.event_store.client.gossip.EndPoint\"\x9a\x02\n\nVNodeState\x12\x10\n\x0cInitializing\x10\x00\x12\x12\n\x0e\x44iscoverLeader\x10\x01\x12\x0b\n\x07Unknown\x10\x02\x12\x0e\n\nPreReplica\x10\x03\x12\x0e\n\nCatchingUp\x10\x04\x12\t\n\x05\x43lone\x10\x05\x12\x0c\n\x08\x46ollower\x10\x06\x12\r\n\tPreLeader\x10\x07\x12\n\n\x06Leader\x10\x08\x12\x0b\n\x07Manager\x10\t\x12\x10\n\x0cShuttingDown\x10\n\x12\x0c\n\x08Shutdown\x10\x0b\x12\x16\n\x12ReadOnlyLeaderless\x10\x0c\x12\x16\n\x12PreReadOnlyReplica\x10\r\x12\x13\n\x0fReadOnlyReplica\x10\x0e\x12\x13\n\x0fResigningLeader\x10\x0f\x32S\n\x06Gossip\x12I\n\x04Read\x12\x19.event_store.client.Empty\x1a&.event_store.client.gossip.ClusterInfoB&\n$com.eventstore.dbclient.proto.gossipb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'gossip_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n$com.eventstore.dbclient.proto.gossip'
  _CLUSTERINFO._serialized_start=57
  _CLUSTERINFO._serialized_end=126
  _ENDPOINT._serialized_start=128
  _ENDPOINT._serialized_end=169
  _MEMBERINFO._serialized_start=172
  _MEMBERINFO._serialized_end=680
  _MEMBERINFO_VNODESTATE._serialized_start=398
  _MEMBERINFO_VNODESTATE._serialized_end=680
  _GOSSIP._serialized_start=682
  _GOSSIP._serialized_end=765
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

import asyncio
import base64
import contextlib
import enum
import itertools
import logging
import random
import urllib.parse
from dataclasses import dataclass
from functools import cmp_to_key
from typing import AsyncContextManager, Optional

import grpc

from esdb.exceptions import DiscoveryError
from esdb.generated.gossip_pb2_grpc import GossipStub
from esdb.generated.persistent_pb2_grpc import PersistentSubscriptionsStub
from esdb.generated.streams_pb2_grpc import StreamsStub
from esdb.gossip import Gossip, Member, State
from esdb.streams import Streams
from esdb.subscriptions import PersistentSubscriptions

logger = logging.getLogger(__name__)


class Preference(enum.Enum):
    LEADER = enum.auto()
    FOLLOWER = enum.auto()
    READ_ONLY_REPLICA = enum.auto()

    @classmethod
    def from_string(cls, s: str) -> Preference:
        return {
            "follower": cls.FOLLOWER,
            "leader": cls.LEADER,
            "readonlyreplica": cls.READ_ONLY_REPLICA,
        }[s.lower()]


@dataclass
class Configuration:
    address: Optional[Member.Endpoint] = None
    gossip_seed: Optional[list[Member.Endpoint]] = None
    username: Optional[str] = None
    password: Optional[str] = None
    disable_tls: bool = False
    node_preference: Preference = Preference.LEADER
    verify_cert: bool = True
    root_cert: Optional[bytes] = None
    max_discovery_attempts: int = 10
    discovery_interval: int = 100
    gossip_timeout: int = 5
    dns_discover: bool = False
    keep_alive_interval: int = 10
    keep_alive_timeout: int = 10
    default_deadline: int = 10


class BasicAuthPlugin(grpc.AuthMetadataPlugin):
    def __init__(self, user: str, password: str) -> None:
        self.__auth = base64.b64encode(f"{user}:{password}".encode())

    def __call__(self, _: grpc.AuthMetadataContext, callback: grpc.AuthMetadataPluginCallback) -> None:
        callback((("authorization", b"Basic " + self.__auth),), None)


@dataclass
class Connection:
    channel: grpc.aio._base_channel.Channel  # type: ignore
    streams: Streams
    subscriptions: PersistentSubscriptions
    gossip: Gossip
    config: Configuration


def pick_node(preference: Preference, members: list[Member]) -> Optional[Member]:
    preference_map = {
        Preference.LEADER: [State.Leader],
        Preference.FOLLOWER: [State.Follower],
        Preference.READ_ONLY_REPLICA: [State.ReadOnlyLeaderless, State.PreReadOnlyReplica, State.ReadOnlyReplica],
    }
    preferred_states = preference_map[preference]

    def _compare(a: Member, b: Member) -> int:
        return (preferred_states.index(b.state) if b.state in preferred_states else -1) - (
            preferred_states.index(a.state) if a.state in preferred_states else -1
        )

    members_ = members.copy()
    random.shuffle(members_)

    member: Optional[Member] = next(
        (
            m
            for m in sorted(members_, key=cmp_to_key(_compare))
            if m.is_alive and m.state in list(itertools.chain(*preference_map.values())) and m.endpoint
        ),
        None,
    )
    if not member or not member.endpoint:
        return None
    return member


def parse_endpoint(s: str) -> Member.Endpoint:
    if ":" in s:
        items = s.split(":")
        if len(items) != 2:
            raise ValueError("Too many colons in a host")
        host, port_str = items
        try:
            port = int(port_str)
        except ValueError:
            raise ValueError(f"{port_str} port is not a number")
        return Member.Endpoint(host, port)

    return Member.Endpoint(s, 2113)


def parse_settings(query: str, c: Configuration) -> None:
    def _str_to_bool(s: str) -> bool:
        return {"true": True, "false": False}[s.lower()]

    for k, v in urllib.parse.parse_qs(query, strict_parsing=True).items():
        if len(v) != 1:
            raise ValueError(f"Too many values for {k}")
        key = k.lower()
        [val] = v
        if key == "discoveryinterval":
            c.discovery_interval = int(val)
        elif key == "gossiptimeout":
            c.gossip_timeout = int(val)
        elif key == "maxdiscoverattempts":
            c.max_discovery_attempts = int(val)
        elif key == "nodepreference":
            c.node_preference = Preference.from_string(val)
        elif key == "keepaliveinterval":
            c.keep_alive_interval = int(val)
        elif key == "keepalivetimeout":
            c.keep_alive_timeout = int(val)
        elif key == "tls":
            c.disable_tls = not _str_to_bool(val)
        elif key == "tlscafile":
            with open(val, "rb") as fh:
                c.root_cert = fh.read()
        elif key == "tlsverifycert":
            c.verify_cert = _str_to_bool(val)
        elif key == "defaultdeadline":
            c.default_deadline = int(val)
        else:
            raise ValueError(f"Invalid option {k}")


def parse_connection_string(connection_string: str) -> Configuration:
    config = Configuration()
    scheme, rest = connection_string.split("://")
    if scheme not in ("esdb", "esdb+discover"):
        raise ValueError("esdb:// or esdb+discover:// scheme is required")

    config.dns_discover = scheme == "esdb+discover"

    if "@" in rest:
        user_info, rest = rest.split("@")
        user_info_items = user_info.split(":")
        if len(user_info_items) != 2:
            raise ValueError("Invalid user credentials")
        user, password = user_info_items
        if not user:
            raise ValueError("Username is required")
        if not password:
            raise ValueError("Password is required")
        config.username, config.password = user, password

    hosts, *queries = rest.split("?")
    endpoints = []
    for host in hosts.split(","):
        endpoints.append(parse_endpoint(host))
    if len(endpoints) == 1:
        config.address = endpoints[0]
    else:
        config.gossip_seed = endpoints

    if queries:
        [settings_query] = queries
        parse_settings(settings_query, config)

    return config


class ESClient:
    def __init__(
        self,
        connection_string: str,
        root_certificates: Optional[bytes] = None,
        private_key: Optional[bytes] = None,
        certificate_chain: Optional[bytes] = None,
    ) -> None:
        self.config = parse_connection_string(connection_string)
        self.options = [
            ("grpc.keepalive_time_ms", self.config.keep_alive_interval * 1000),
            ("grpc.keepalive_timeout_ms", self.config.keep_alive_timeout * 1000),
        ]

        if not self.config.disable_tls:
            self.channel_credentials = grpc.ssl_channel_credentials(
                root_certificates=root_certificates or self.config.root_cert,
                private_key=private_key,
                certificate_chain=certificate_chain,
            )

        if self.config.username and self.config.password:
            self.call_credentials = grpc.metadata_call_credentials(
                BasicAuthPlugin(self.config.username, self.config.password), name="auth"
            )

    def _create_channel(self, endpoint: Member.Endpoint) -> grpc.aio.Channel:  # type: ignore
        if self.config.disable_tls:
            return grpc.aio.insecure_channel(  # type: ignore
                f"{endpoint.address}:{endpoint.port}", options=self.options
            )
        assert self.channel_credentials
        credentials = (
            grpc.composite_channel_credentials(self.channel_credentials, self.call_credentials)
            if self.call_credentials
            else self.channel_credentials
        )
        return grpc.aio.secure_channel(  # type: ignore
            f"{endpoint.address}:{endpoint.port}", credentials, self.options
        )

    @contextlib.asynccontextmanager  # type: ignore
    async def connect(self) -> AsyncContextManager[Connection]:  # type: ignore
        if self.config.dns_discover:
            endpoint = await self.discover_endpoint()
        else:
            assert self.config.address
            endpoint = self.config.address

        async with self._create_channel(endpoint) as channel:
            yield Connection(
                channel=channel,
                streams=Streams(StreamsStub(channel)),
                subscriptions=PersistentSubscriptions(PersistentSubscriptionsStub(channel)),
                gossip=Gossip(GossipStub(channel)),
                config=self.config,
            )

    async def discover_endpoint(self) -> Member.Endpoint:
        for attempt in range(1, self.config.max_discovery_attempts + 1):
            candidates = (
                self.config.gossip_seed.copy() if self.config.gossip_seed else [self.config.address]  # type: ignore
            )
            random.shuffle(candidates)
            logger.info(
                "Starting discovery attempt %s on %s", attempt, ",".join(f"{c.address}:{c.port}" for c in candidates)
            )
            for candidate in candidates:
                async with self._create_channel(candidate) as chan:
                    gossip = Gossip(GossipStub(chan))
                    members = await gossip.get_members(self.config.gossip_timeout)
                    if pick := pick_node(self.config.node_preference, members):
                        assert pick.endpoint
                        logger.info(
                            "Discovered %s node at %s (Preference: %s)",
                            pick.state.name,
                            f"{pick.endpoint.address}:{pick.endpoint.port}",
                            self.config.node_preference.name,
                        )
                        return pick.endpoint

            await asyncio.sleep(self.config.discovery_interval)
        raise DiscoveryError(f"Discovery failed after {self.config.max_discovery_attempts} attempt(s)")



from __future__ import annotations

import asyncio
from typing import AsyncIterable, AsyncIterator, Optional

from esdb.generated.persistent_pb2 import (
    CreateReq,
    CreateResp,
    DeleteReq,
    GetInfoReq,
    GetInfoResp,
    ListReq,
    ListResp,
    ReadReq,
    ReadResp,
    UpdateReq,
    UpdateResp,
)
from esdb.generated.persistent_pb2_grpc import PersistentSubscriptionsStub
from esdb.generated.shared_pb2 import UUID, Empty, StreamIdentifier
from esdb.shared import Filter
from esdb.subscriptions.types import (
    Event,
    NackAction,
    SubscriptionInfo,
    SubscriptionSettings,
)


class Subscription:
    def __init__(
        self, group_name: str, buffer_size: int, stream: Optional[str], stub: PersistentSubscriptionsStub
    ) -> None:
        self.stream = stream
        self.group_name = group_name
        self.buffer_size = buffer_size
        self.send_queue: asyncio.Queue = asyncio.Queue()
        self.stub = stub
        self.subscription_id: Optional[str] = None

    async def __aiter__(self) -> AsyncIterable[Event]:
        read_request = ReadReq(
            options=ReadReq.Options(
                stream_identifier=StreamIdentifier(stream_name=self.stream.encode()) if self.stream else None,
                all=Empty() if self.stream is None else None,
                group_name=self.group_name,
                buffer_size=self.buffer_size,
                uuid_option=ReadReq.Options.UUIDOption(structured=Empty(), string=Empty()),
            ),
            ack=None,
            nack=None,
        )
        await self.send_queue.put(read_request)

        async def queue_iter():
            while True:
                yield await self.send_queue.get()
                self.send_queue.task_done()

        reader: AsyncIterator[ReadResp] = self.stub.Read(queue_iter())
        async for response in reader:
            if not self.subscription_id and response.WhichOneof("content") == "subscription_confirmation":
                self.subscription_id = response.subscription_confirmation.subscription_id
                continue
            yield Event.from_read_response_event(response.event)

    async def ack(self, events: list[Event]) -> None:
        assert self.subscription_id, "Nothing to ack, not reading from a subscription yet"
        await self.send_queue.put(
            ReadReq(
                ack=ReadReq.Ack(
                    id=self.subscription_id.encode(),
                    ids=(UUID(string=evt.id) for evt in events),
                )
            )
        )
        await self.send_queue.join()

    async def nack(self, events: list[Event], action: NackAction, reason: Optional[str] = None) -> None:
        assert self.subscription_id, "Nothing to nack, not reading from a subscription yet"
        await self.send_queue.put(
            ReadReq(
                nack=ReadReq.Nack(
                    id=self.subscription_id.encode(),
                    ids=(UUID(string=evt.id) for evt in events),
                    action=action.value,
                    reason=reason or "",
                )
            )
        )


class PersistentSubscriptions:
    def __init__(self, stub: PersistentSubscriptionsStub) -> None:
        self._stub = stub

    async def create_stream_subscription(
        self, stream: str, group_name: str, settings: SubscriptionSettings, backwards: bool = False
    ) -> None:
        stream_identifier = StreamIdentifier(stream_name=stream.encode())
        create_request = CreateReq(
            options=CreateReq.Options(
                stream=CreateReq.StreamOptions(
                    stream_identifier=stream_identifier,
                    start=None if backwards else Empty(),
                    end=Empty() if backwards else None,
                ),
                stream_identifier=stream_identifier,
                group_name=group_name,
                settings=settings.to_protobuf(CreateReq.Settings),
            )
        )
        response: CreateResp = await self._stub.Create(create_request)
        assert isinstance(response, CreateResp), f"Expected {CreateResp} got {response.__class__}"

    async def create_all_subscription(
        self,
        group_name: str,
        settings: SubscriptionSettings,
        backwards: bool = False,
        filter_by: Optional[Filter] = None,
    ) -> None:
        create_request = CreateReq(
            options=CreateReq.Options(
                group_name=group_name,
                settings=settings.to_protobuf(CreateReq.Settings),
                all=CreateReq.AllOptions(
                    # position=CreateReq.Position(), TODO: deal with position
                    start=None if backwards else Empty(),
                    end=Empty() if backwards else None,
                    no_filter=Empty() if filter_by is None else None,
                    filter=filter_by.to_protobuf(CreateReq.AllOptions.FilterOptions) if filter_by else None,
                ),
            )
        )
        response: CreateResp = await self._stub.Create(create_request)
        assert isinstance(response, CreateResp), f"Expected {CreateResp} got {response.__class__}"

    def subscribe(
        self,
        group_name: str,
        buffer_size: int,
        stream: Optional[str] = None,
    ) -> Subscription:
        return Subscription(stream=stream, group_name=group_name, buffer_size=buffer_size, stub=self._stub)

    async def update_stream_subscription(
        self, stream: str, group_name: str, settings: SubscriptionSettings, backwards: bool = False
    ) -> None:
        stream_identifier = StreamIdentifier(stream_name=stream.encode())
        update_request = UpdateReq(
            options=UpdateReq.Options(
                stream=UpdateReq.StreamOptions(
                    stream_identifier=stream_identifier,
                    start=None if backwards else Empty(),
                    end=Empty() if backwards else None,
                ),
                stream_identifier=stream_identifier,
                group_name=group_name,
                settings=settings.to_protobuf(UpdateReq.Settings),
            )
        )
        response: UpdateResp = await self._stub.Update(update_request)
        assert isinstance(response, UpdateResp), f"Expected {UpdateResp} got {response.__class__}"

    async def update_all_subscription(
        self, group_name: str, settings: SubscriptionSettings, backwards: bool = False
    ) -> None:
        update_request = UpdateReq(
            options=UpdateReq.Options(
                group_name=group_name,
                settings=settings.to_protobuf(UpdateReq.Settings),
                all=UpdateReq.AllOptions(
                    # position=CreateReq.Position(), TODO: deal with position
                    start=None if backwards else Empty(),
                    end=Empty() if backwards else None,
                ),
            )
        )
        response: UpdateResp = await self._stub.Update(update_request)
        assert isinstance(response, UpdateResp), f"Expected {UpdateResp} got {response.__class__}"

    async def get_info(self, group_name: str, stream: Optional[str] = None) -> SubscriptionInfo:
        info_request = GetInfoReq(
            options=GetInfoReq.Options(
                group_name=group_name,
                stream_identifier=StreamIdentifier(stream_name=stream.encode()) if stream else None,
                all=Empty() if stream is None else None,
            )
        )
        response: GetInfoResp = await self._stub.GetInfo(info_request)
        return SubscriptionInfo.from_protobuf(response.subscription_info)

    async def list(self, stream: Optional[str] = None) -> list[SubscriptionInfo]:
        request = ListReq(
            options=ListReq.Options(
                list_all_subscriptions=Empty() if stream is None else None,
                list_for_stream=None
                if stream is None
                else ListReq.StreamOption(stream=StreamIdentifier(stream_name=stream.encode())),
            )
        )
        response: ListResp = await self._stub.List(request)
        return [SubscriptionInfo.from_protobuf(sub) for sub in response.subscriptions]

    async def delete(self, group_name: str, stream: Optional[str] = None) -> None:
        request = DeleteReq(
            options=DeleteReq.Options(
                group_name=group_name,
                all=Empty() if stream is None else None,
                stream_identifier=StreamIdentifier(stream_name=stream.encode()) if stream is not None else None,
            )
        )
        await self._stub.Delete(request)



import logging
import uuid
from typing import AsyncIterable, Iterable, Optional, Union

import grpc
from google.protobuf.duration_pb2 import Duration
from google.protobuf.empty_pb2 import Empty as GEmpty

from esdb.exceptions import ClientException, StreamNotFound, WrongExpectedVersion
from esdb.generated.shared_pb2 import UUID, Empty, StreamIdentifier
from esdb.generated.streams_pb2 import (
    AppendReq,
    AppendResp,
    BatchAppendReq,
    DeleteReq,
    DeleteResp,
    ReadReq,
    ReadResp,
    TombstoneReq,
    TombstoneResp,
)
from esdb.generated.streams_pb2_grpc import StreamsStub
from esdb.shared import Filter
from esdb.streams.types import (
    AppendResult,
    BatchAppendResult,
    Checkpoint,
    DeleteResult,
    Message,
    ReadEvent,
    StreamState,
    SubscriptionConfirmed,
    TombstoneResult,
)

logger = logging.getLogger(__name__)


class Streams:
    def __init__(self, stub: StreamsStub) -> None:
        self._stub = stub

    async def append(
        self,
        stream: str,
        event_type: str,
        data: Union[dict, bytes],
        stream_state: StreamState = StreamState.ANY,
        revision: Optional[int] = None,
        custom_metadata: Optional[dict] = None,
    ) -> AppendResult:

        options = AppendReq.Options(
            stream_identifier=StreamIdentifier(stream_name=stream.encode()),
            **{stream_state.value: Empty()},  # type: ignore
        )
        if revision is not None:
            # Append at specified revision
            options.revision = revision
        requests = [
            AppendReq(options=options),
            AppendReq(
                proposed_message=Message(
                    event_type=event_type,
                    data=data,
                    custom_metadata=custom_metadata,
                ).to_protobuf(AppendReq.ProposedMessage)
            ),
        ]
        response: AppendResp = await self._stub.Append(requests)
        if response.WhichOneof("result") == "wrong_expected_version":
            raise WrongExpectedVersion(response.wrong_expected_version)
        return AppendResult.from_response(response)

    async def _get_events(self, responses: AsyncIterable[ReadResp]) -> AsyncIterable[ReadEvent]:
        """Filter out the responses from read requests, since we only care about read events."""
        async for response in responses:
            result = self._process_read_response(response)
            if isinstance(result, ReadEvent):
                yield result
            elif isinstance(result, SubscriptionConfirmed):
                logger.info(f"Subscription {result.subscription_id} confirmed")
            elif isinstance(result, Checkpoint):
                logger.info(
                    f"Checkpoint commit position: {result.commit_position}, "
                    f"prepare position: {result.prepare_position}"
                )

    @staticmethod
    def _process_read_response(response: ReadResp) -> Union[ReadEvent, SubscriptionConfirmed, Checkpoint]:
        content = response.WhichOneof("content")
        if content == "event":
            return ReadEvent.from_response(response)
        if content == "confirmation":
            return SubscriptionConfirmed(response.confirmation.subscription_id)
        if content == "checkpoint":
            return Checkpoint(
                commit_position=response.checkpoint.commit_position,
                prepare_position=response.checkpoint.prepare_position,
            )
        if content == "stream_not_found":
            raise StreamNotFound(response.stream_not_found)
        raise ClientException(f"Got unexpected response {content}: {getattr(response, content)}")  # type: ignore

    async def read(
        self,
        stream: str,
        count: Optional[int] = None,
        backwards: bool = False,
        revision: Optional[int] = None,
        subscribe: bool = False,
    ) -> AsyncIterable[ReadEvent]:
        assert (count is not None) ^ subscribe, "count or subscribe is required"

        stream_options = ReadReq.Options.StreamOptions(
            stream_identifier=StreamIdentifier(stream_name=stream.encode()),
            start=None if backwards else Empty(),
            end=Empty() if backwards else None,
        )
        if revision is not None:
            stream_options.revision = revision

        options = ReadReq.Options(
            stream=stream_options,
            all=None,
            read_direction=ReadReq.Options.Backwards if backwards else ReadReq.Options.Forwards,
            resolve_links=True,  # Resolve to actual data instead of a link when reading from projection
            subscription=ReadReq.Options.SubscriptionOptions() if subscribe else None,
            filter=None,
            no_filter=Empty(),
            uuid_option=ReadReq.Options.UUIDOption(structured=Empty(), string=Empty()),
        )
        if count is not None:
            options.count = count

        async for response in self._get_events(self._stub.Read(ReadReq(options=options))):
            yield response

    async def read_all(
        self, count: Optional[int] = None, backwards=False, subscribe=False, filter_by: Optional[Filter] = None
    ) -> AsyncIterable[ReadEvent]:

        assert subscribe ^ (count is not None), "subscribe and count are mutually exclusive arguments"
        request = ReadReq(
            options=ReadReq.Options(
                stream=None,
                all=ReadReq.Options.AllOptions(
                    start=None if backwards else Empty(),
                    end=Empty() if backwards else None,
                    # position=ReadReq.Options.Position()  # TODO: implement position
                ),
                read_direction=ReadReq.Options.Backwards if backwards else ReadReq.Options.Forwards,
                resolve_links=True,
                subscription=ReadReq.Options.SubscriptionOptions() if subscribe else None,
                filter=filter_by.to_protobuf(ReadReq.Options.FilterOptions) if filter_by else None,
                no_filter=None if filter_by else Empty(),
                uuid_option=ReadReq.Options.UUIDOption(structured=Empty(), string=Empty()),
            )
        )
        if count is not None:
            request.options.count = count
        if subscribe and filter_by:
            assert (
                filter_by.checkpoint_interval_multiplier is not None
            ), "checkpoint_interval_multiplier is required when subscribing"
        async for response in self._get_events(self._stub.Read(request)):
            yield response

    async def delete(
        self, stream: str, stream_state: StreamState = StreamState.ANY, revision: Optional[int] = None
    ) -> Optional[DeleteResult]:
        request = DeleteReq(
            options=DeleteReq.Options(
                stream_identifier=StreamIdentifier(stream_name=stream.encode()),
                **{stream_state.value: Empty()},  # type: ignore
            )
        )
        if revision is not None:
            request.options.revision = revision
        try:
            response: DeleteResp = await self._stub.Delete(request)
        except grpc.aio._call.AioRpcError as err:  # type: ignore
            raise ClientException(f"Delete failed: {err.details()}") from err
        has_position = response.WhichOneof("position_option") == "position"
        return DeleteResult.from_response(response) if has_position else None

    async def tombstone(
        self, stream: str, stream_state: StreamState = StreamState.ANY, revision: Optional[int] = None
    ) -> Optional[TombstoneResult]:
        request = TombstoneReq(
            options=TombstoneReq.Options(
                stream_identifier=StreamIdentifier(stream_name=stream.encode()),
                **{stream_state.value: Empty()},  # type: ignore
            )
        )
        if revision is not None:
            request.options.revision = revision
        response: TombstoneResp = await self._stub.Tombstone(request)
        has_position = response.WhichOneof("position_option") == "position"
        return TombstoneResult.from_response(response) if has_position else None

    async def batch_append(
        self,
        stream: str,
        messages: Iterable[Message],
        stream_state: Optional[StreamState] = None,
        correlation_id: Optional[uuid.UUID] = None,
        deadline_ms: Optional[int] = None,
        stream_position: Optional[int] = None,
    ) -> BatchAppendResult:
        if stream_position is not None and stream_state is not None:
            raise ValueError("stream_position can't be used with stream_state")
        if stream_position is None and stream_state is None:
            stream_state = StreamState.ANY
        correlation_id_ = UUID(string=str(correlation_id or uuid.uuid4()))
        stream_opts = {stream_state.value: GEmpty()} if stream_state else {}
        options_req = BatchAppendReq(
            correlation_id=correlation_id_,
            options=BatchAppendReq.Options(
                stream_identifier=StreamIdentifier(stream_name=stream.encode()),
                deadline=Duration(nanos=deadline_ms * 1000000) if deadline_ms is not None else None,
                **stream_opts,  # type: ignore
            ),
            is_final=False,
        )
        if stream_position is not None:
            options_req.options.stream_position = stream_position

        requests = [
            options_req,
            BatchAppendReq(
                correlation_id=correlation_id_,
                proposed_messages=(m.to_protobuf(BatchAppendReq.ProposedMessage) for m in messages),
                is_final=True,
            ),
        ]
        result = None
        async for response in self._stub.BatchAppend(requests):
            if response.WhichOneof("result") == "error":
                # For some reason ES uses google.rpc.Status here instead of more meaningful error.
                raise ClientException(f"Append failed with {response.error.message} and code {response.error.code}")
            result = BatchAppendResult.from_response(response)
            # TODO: Handle streaming of multiple batches and responses
            break
        assert result
        return result



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import users_pb2 as users__pb2


class UsersStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Create = channel.unary_unary(
                '/event_store.client.users.Users/Create',
                request_serializer=users__pb2.CreateReq.SerializeToString,
                response_deserializer=users__pb2.CreateResp.FromString,
                )
        self.Update = channel.unary_unary(
                '/event_store.client.users.Users/Update',
                request_serializer=users__pb2.UpdateReq.SerializeToString,
                response_deserializer=users__pb2.UpdateResp.FromString,
                )
        self.Delete = channel.unary_unary(
                '/event_store.client.users.Users/Delete',
                request_serializer=users__pb2.DeleteReq.SerializeToString,
                response_deserializer=users__pb2.DeleteResp.FromString,
                )
        self.Disable = channel.unary_unary(
                '/event_store.client.users.Users/Disable',
                request_serializer=users__pb2.DisableReq.SerializeToString,
                response_deserializer=users__pb2.DisableResp.FromString,
                )
        self.Enable = channel.unary_unary(
                '/event_store.client.users.Users/Enable',
                request_serializer=users__pb2.EnableReq.SerializeToString,
                response_deserializer=users__pb2.EnableResp.FromString,
                )
        self.Details = channel.unary_stream(
                '/event_store.client.users.Users/Details',
                request_serializer=users__pb2.DetailsReq.SerializeToString,
                response_deserializer=users__pb2.DetailsResp.FromString,
                )
        self.ChangePassword = channel.unary_unary(
                '/event_store.client.users.Users/ChangePassword',
                request_serializer=users__pb2.ChangePasswordReq.SerializeToString,
                response_deserializer=users__pb2.ChangePasswordResp.FromString,
                )
        self.ResetPassword = channel.unary_unary(
                '/event_store.client.users.Users/ResetPassword',
                request_serializer=users__pb2.ResetPasswordReq.SerializeToString,
                response_deserializer=users__pb2.ResetPasswordResp.FromString,
                )


class UsersServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Update(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Delete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Disable(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Enable(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Details(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ChangePassword(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetPassword(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_UsersServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Create': grpc.unary_unary_rpc_method_handler(
                    servicer.Create,
                    request_deserializer=users__pb2.CreateReq.FromString,
                    response_serializer=users__pb2.CreateResp.SerializeToString,
            ),
            'Update': grpc.unary_unary_rpc_method_handler(
                    servicer.Update,
                    request_deserializer=users__pb2.UpdateReq.FromString,
                    response_serializer=users__pb2.UpdateResp.SerializeToString,
            ),
            'Delete': grpc.unary_unary_rpc_method_handler(
                    servicer.Delete,
                    request_deserializer=users__pb2.DeleteReq.FromString,
                    response_serializer=users__pb2.DeleteResp.SerializeToString,
            ),
            'Disable': grpc.unary_unary_rpc_method_handler(
                    servicer.Disable,
                    request_deserializer=users__pb2.DisableReq.FromString,
                    response_serializer=users__pb2.DisableResp.SerializeToString,
            ),
            'Enable': grpc.unary_unary_rpc_method_handler(
                    servicer.Enable,
                    request_deserializer=users__pb2.EnableReq.FromString,
                    response_serializer=users__pb2.EnableResp.SerializeToString,
            ),
            'Details': grpc.unary_stream_rpc_method_handler(
                    servicer.Details,
                    request_deserializer=users__pb2.DetailsReq.FromString,
                    response_serializer=users__pb2.DetailsResp.SerializeToString,
            ),
            'ChangePassword': grpc.unary_unary_rpc_method_handler(
                    servicer.ChangePassword,
                    request_deserializer=users__pb2.ChangePasswordReq.FromString,
                    response_serializer=users__pb2.ChangePasswordResp.SerializeToString,
            ),
            'ResetPassword': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetPassword,
                    request_deserializer=users__pb2.ResetPasswordReq.FromString,
                    response_serializer=users__pb2.ResetPasswordResp.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'event_store.client.users.Users', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Users(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Create(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.users.Users/Create',
            users__pb2.CreateReq.SerializeToString,
            users__pb2.CreateResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Update(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.users.Users/Update',
            users__pb2.UpdateReq.SerializeToString,
            users__pb2.UpdateResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Delete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.users.Users/Delete',
            users__pb2.DeleteReq.SerializeToString,
            users__pb2.DeleteResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Disable(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.users.Users/Disable',
            users__pb2.DisableReq.SerializeToString,
            users__pb2.DisableResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Enable(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.users.Users/Enable',
            users__pb2.EnableReq.SerializeToString,
            users__pb2.EnableResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Details(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/event_store.client.users.Users/Details',
            users__pb2.DetailsReq.SerializeToString,
            users__pb2.DetailsResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ChangePassword(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.users.Users/ChangePassword',
            users__pb2.ChangePasswordReq.SerializeToString,
            users__pb2.ChangePasswordResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetPassword(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.users.Users/ResetPassword',
            users__pb2.ResetPasswordReq.SerializeToString,
            users__pb2.ResetPasswordResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from __future__ import annotations

import datetime
import enum
from dataclasses import dataclass
from typing import Optional

from esdb.generated.gossip_pb2 import ClusterInfo, MemberInfo
from esdb.generated.gossip_pb2_grpc import GossipStub
from esdb.generated.shared_pb2 import Empty


class State(enum.IntEnum):
    Initializing = 0
    DiscoverLeader = enum.auto()
    Unknown = enum.auto()
    PreReplica = enum.auto()
    CatchingUp = enum.auto()
    Clone = enum.auto()
    Follower = enum.auto()
    PreLeader = enum.auto()
    Leader = enum.auto()
    Manager = enum.auto()
    ShuttingDown = enum.auto()
    Shutdown = enum.auto()
    ReadOnlyLeaderless = enum.auto()
    PreReadOnlyReplica = enum.auto()
    ReadOnlyReplica = enum.auto()
    ResigningLeader = enum.auto()


@dataclass
class Member:
    @dataclass
    class Endpoint:
        address: str
        port: int

    timestamp: datetime.datetime
    state: State
    is_alive: bool
    endpoint: Optional[Endpoint]

    @classmethod
    def from_protobuf(cls, m: MemberInfo) -> Member:
        return cls(
            timestamp=datetime.datetime.fromtimestamp(m.time_stamp / 10000000, datetime.timezone.utc),
            state=State(m.state),
            is_alive=m.is_alive,
            endpoint=cls.Endpoint(m.http_end_point.address, m.http_end_point.port)
            if m.http_end_point.address
            else None,
        )


class Gossip:
    def __init__(self, stub: GossipStub) -> None:
        self._stub = stub

    async def get_members(self, timeout: int) -> list[Member]:
        info: ClusterInfo = await self._stub.Read(Empty(), timeout=timeout)
        return [Member.from_protobuf(m) for m in info.members]



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import projections_pb2 as projections__pb2
from . import shared_pb2 as shared__pb2


class ProjectionsStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Create = channel.unary_unary(
                '/event_store.client.projections.Projections/Create',
                request_serializer=projections__pb2.CreateReq.SerializeToString,
                response_deserializer=projections__pb2.CreateResp.FromString,
                )
        self.Update = channel.unary_unary(
                '/event_store.client.projections.Projections/Update',
                request_serializer=projections__pb2.UpdateReq.SerializeToString,
                response_deserializer=projections__pb2.UpdateResp.FromString,
                )
        self.Delete = channel.unary_unary(
                '/event_store.client.projections.Projections/Delete',
                request_serializer=projections__pb2.DeleteReq.SerializeToString,
                response_deserializer=projections__pb2.DeleteResp.FromString,
                )
        self.Statistics = channel.unary_stream(
                '/event_store.client.projections.Projections/Statistics',
                request_serializer=projections__pb2.StatisticsReq.SerializeToString,
                response_deserializer=projections__pb2.StatisticsResp.FromString,
                )
        self.Disable = channel.unary_unary(
                '/event_store.client.projections.Projections/Disable',
                request_serializer=projections__pb2.DisableReq.SerializeToString,
                response_deserializer=projections__pb2.DisableResp.FromString,
                )
        self.Enable = channel.unary_unary(
                '/event_store.client.projections.Projections/Enable',
                request_serializer=projections__pb2.EnableReq.SerializeToString,
                response_deserializer=projections__pb2.EnableResp.FromString,
                )
        self.Reset = channel.unary_unary(
                '/event_store.client.projections.Projections/Reset',
                request_serializer=projections__pb2.ResetReq.SerializeToString,
                response_deserializer=projections__pb2.ResetResp.FromString,
                )
        self.State = channel.unary_unary(
                '/event_store.client.projections.Projections/State',
                request_serializer=projections__pb2.StateReq.SerializeToString,
                response_deserializer=projections__pb2.StateResp.FromString,
                )
        self.Result = channel.unary_unary(
                '/event_store.client.projections.Projections/Result',
                request_serializer=projections__pb2.ResultReq.SerializeToString,
                response_deserializer=projections__pb2.ResultResp.FromString,
                )
        self.RestartSubsystem = channel.unary_unary(
                '/event_store.client.projections.Projections/RestartSubsystem',
                request_serializer=shared__pb2.Empty.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )


class ProjectionsServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Update(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Delete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Statistics(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Disable(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Enable(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Reset(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def State(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Result(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RestartSubsystem(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ProjectionsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Create': grpc.unary_unary_rpc_method_handler(
                    servicer.Create,
                    request_deserializer=projections__pb2.CreateReq.FromString,
                    response_serializer=projections__pb2.CreateResp.SerializeToString,
            ),
            'Update': grpc.unary_unary_rpc_method_handler(
                    servicer.Update,
                    request_deserializer=projections__pb2.UpdateReq.FromString,
                    response_serializer=projections__pb2.UpdateResp.SerializeToString,
            ),
            'Delete': grpc.unary_unary_rpc_method_handler(
                    servicer.Delete,
                    request_deserializer=projections__pb2.DeleteReq.FromString,
                    response_serializer=projections__pb2.DeleteResp.SerializeToString,
            ),
            'Statistics': grpc.unary_stream_rpc_method_handler(
                    servicer.Statistics,
                    request_deserializer=projections__pb2.StatisticsReq.FromString,
                    response_serializer=projections__pb2.StatisticsResp.SerializeToString,
            ),
            'Disable': grpc.unary_unary_rpc_method_handler(
                    servicer.Disable,
                    request_deserializer=projections__pb2.DisableReq.FromString,
                    response_serializer=projections__pb2.DisableResp.SerializeToString,
            ),
            'Enable': grpc.unary_unary_rpc_method_handler(
                    servicer.Enable,
                    request_deserializer=projections__pb2.EnableReq.FromString,
                    response_serializer=projections__pb2.EnableResp.SerializeToString,
            ),
            'Reset': grpc.unary_unary_rpc_method_handler(
                    servicer.Reset,
                    request_deserializer=projections__pb2.ResetReq.FromString,
                    response_serializer=projections__pb2.ResetResp.SerializeToString,
            ),
            'State': grpc.unary_unary_rpc_method_handler(
                    servicer.State,
                    request_deserializer=projections__pb2.StateReq.FromString,
                    response_serializer=projections__pb2.StateResp.SerializeToString,
            ),
            'Result': grpc.unary_unary_rpc_method_handler(
                    servicer.Result,
                    request_deserializer=projections__pb2.ResultReq.FromString,
                    response_serializer=projections__pb2.ResultResp.SerializeToString,
            ),
            'RestartSubsystem': grpc.unary_unary_rpc_method_handler(
                    servicer.RestartSubsystem,
                    request_deserializer=shared__pb2.Empty.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'event_store.client.projections.Projections', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Projections(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Create(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.projections.Projections/Create',
            projections__pb2.CreateReq.SerializeToString,
            projections__pb2.CreateResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Update(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.projections.Projections/Update',
            projections__pb2.UpdateReq.SerializeToString,
            projections__pb2.UpdateResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Delete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.projections.Projections/Delete',
            projections__pb2.DeleteReq.SerializeToString,
            projections__pb2.DeleteResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Statistics(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/event_store.client.projections.Projections/Statistics',
            projections__pb2.StatisticsReq.SerializeToString,
            projections__pb2.StatisticsResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Disable(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.projections.Projections/Disable',
            projections__pb2.DisableReq.SerializeToString,
            projections__pb2.DisableResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Enable(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.projections.Projections/Enable',
            projections__pb2.EnableReq.SerializeToString,
            projections__pb2.EnableResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Reset(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.projections.Projections/Reset',
            projections__pb2.ResetReq.SerializeToString,
            projections__pb2.ResetResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def State(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.projections.Projections/State',
            projections__pb2.StateReq.SerializeToString,
            projections__pb2.StateResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Result(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.projections.Projections/Result',
            projections__pb2.ResultReq.SerializeToString,
            projections__pb2.ResultResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RestartSubsystem(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.projections.Projections/RestartSubsystem',
            shared__pb2.Empty.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
